/* AMX Mod functions
 *
 * (c) Copyright 2002, OLO
 * This file is provided as is (no warranties).
 */

#if defined _amxmod_included
	#endinput
#endif
#define _amxmod_included

#include <core>
#include <float>
#include <amxconst>
#include <string>
#include <file>

/* Function is called just after server activation.
* Good place for configuration loading, commands and cvars registration. */
forward plugin_init();

/* Function called before plugin unloading (server deactivation) */
forward plugin_end();

/* Whenever player info is changed, this function is called. */
forward client_infochanged(id);

/* Called on client connection. */
forward client_connect(id);

/* Called when client is disconnecting from server. */
forward client_disconnect(id);

/* Called when client is sending command. */
forward client_command(id);

/* Called when client is entering to the game. */
forward client_putinserver(id);

/* Use here model_precache() and sound_precache() functions. */
forward plugin_precache();

/* Sets informations about plugin. */
native register_plugin(plugin_name[],version[],author[]);

/* Returns info about plugin by given index. */
native get_plugin(index,filename[],len1,name[],len2,version[],len3,author[],len4,status[],len5);

/* Returns number of all loaded plugins. */
native get_pluginsnum();

/* Precache model. Can be used only in plugin_precache() function.*/
native precache_model(name[]);

/* Precache sound. Can be used only in plugin_precache() function.*/
native precache_sound(name[]);

/* Sets who can listen who. Function returns 0
* if for some reasons this setting can't be done. */
native set_user_listening(receiver,sender,listen);

/* Returns 1 if receiver hears sender via voice communication. */
native get_user_listening(receiver,sender);

/* Sets info for player. */
native set_user_info(index,info[],value[]);

/* Gets info from player. */
native get_user_info(index,info[],output[],len);

/* Sets info for server. */
native set_localinfo(info[],value[]);

/* Gets info from server. */
native get_localinfo(info[],output[],len);

/* Shows text in MOTD window. When there is no header, the MOTD title 
* will be the name of server. If message is filename, then the contents
* of this file will be displayed as MOTD. */
native show_motd(player,message[],header[]="");

/* Sends message to player. Set index to 0 to send text globaly. */
native client_print(index,type,message[],{Float,_}:...);

/* Sends message to player by engine. Set index to 0 to send text globaly. */
native engclient_print(player,type,message[],{Float,_}:...);

/* Registers event on which the given function will be called
* Flags:
* "a" - global event.
* "b" - specified.
* "c" - send only once when repeated to other players.
* "d" - call if is send to dead player.
* "e" - to alive.
* Examples for conditions:
* "2=c4" - 2nd parameter of message must be sting "c4".
* "3>10" - 3rd parameter must be greater then 10.
* "3!4" - 3rd must be different from 4.
* "2&Buy" - 2nd parameter of message must contain "Buy" substring.
* "2!Buy" - 2nd parameter of message can't contain "Buy" substring. */
native register_event(event[],function[],flags[],cond[]="", ... );

/* Sets format for hudmessage. */
native set_hudmessage(red=200, green=100, blue=0, Float:x=-1.0, Float:y=0.35, effects=0, Float:fxtime=6.0, Float:holdtime=12.0, Float:fadeintime=0.1, Float:fadeouttime=0.2,channel=4);

/* Displays HUD message to given player. */
native show_hudmessage(index,message[],{Float,_}:...);

/* Displays menu. Keys have bit values (key 1 is (1<<0), key 5 is (1<<4) etc.). */
native show_menu(index,keys,menu[], time = -1);

/* Gets value from client messages.
* When you are asking for string the array and length is needed (read_data(2,name,len)).
* Integer is returned by function (new me = read_data(3)).
* Float is set in second parameter (read_data(3,value)). */
native read_data(value, {Float,_}:... );

/* Prints message to server console.
* You may use text formating (f.e. server_print("%-32s %.2f!","hello",7.345)) */
native server_print(message[], {Float,_}:...);

/* Returns 1 or 0. */
native is_map_valid(mapname[]);

/* Returns 1 or 0. */
native is_user_bot(index);

/* Returns 1 or 0. */
native is_user_connected(index);

/* Returns 1 or 0. */
native is_user_alive(index);

/* Sets player godmode. If you want to disable godmode set only first parameter. */
native set_user_godmode(index,godmode = 0);

/* Returns 1 if godmode is set. */
native get_user_godmode(index);

/* Sets player noclip. If you want to disable noclip set only first parameter. */
native set_user_noclip(index,noclip = 0);

/* Returns 1 if noclip is set. */
native get_user_noclip(index);

/* Sets player frags. */
native set_user_frags(index,frags);

/* Sets player armor. */
native set_user_armor(index,armor);

/* Sets player health. */
native set_user_health(index, health);

/* If player is not attacked function returns 0, in other
* case returns index of attacking player. On second and third
* parameter you may get info about weapon and body hit place. */
native get_user_attacker(index,...);

/* If player doesn't hit at anything function returns 0.0,
* in other case the distance between hit point and player is returned.
* If player is aiming at another player then the id and part of body are set. */
native Float:get_user_aiming(index,&id,&body,dist=9999);

/* Returns player frags. */
native get_user_frags(index);

/* Returns player deaths. */
native get_user_deaths(index);

/* Returns player armor. */
native get_user_armor(index);

/* Returns player health. */
native get_user_health(index);

/* Returns index. */
native get_user_index(name[]);

/* Returns ip. */
native get_user_ip(index,ip[],len, without_port = 0);

/* Returns id of currently carried weapon. Gets also 
* ammount of ammo in clip and backpack. */
native get_user_weapon(index,&clip,&ammo);

/* Gets ammo and clip from current weapon. */
native get_user_ammo(index,weapon,&clip,&ammo);

/* Coverts number from range 0 - 999 to words. */
native num_to_word(num,output[],len);

/* Returns team id. When length is greater then 0
* then the name of team is set. */
native get_user_team(index, team[]="", len = 0);

/* Returns player playing time in seconds. 
* If flag is set then result is without connection time.
*/
native get_user_time(index, flag = 0);

/* Gets ping and loss at current time. */
native get_user_ping(index, &ping, &loss);

/* Gets origin from player.
* Modes:
* 0 - current position.
* 1 - position from eyes (weapon aiming).
* 2 - end position from player position.
* 3 - end position from eyes (hit point for weapon). */
native get_user_origin(index, origin[3], mode = 0);

/* Move player to origin. */
native set_user_origin(index, origin[3]);

/* Returns all carried weapons as bit sum. Gets
* also theirs indexes. */
native get_user_weapons(index,weapons[32],&num);

/* Returns weapon name. */
native get_weaponname(id,weapon[],len);

/* Returns player name. */
native get_user_name(index,name[],len);

/* Gets player authid. */
native get_user_authid(index, authid[] ,len);

/* Returns player wonid. */
native get_user_wonid(index);

/* Returns player userid. */
native get_user_userid(index);

/* Slaps player with given power. */
native user_slap(index,power);

/* Kills player. When frag var is set to 1 then death won't decrase frags. */
native user_kill(index,frag=0);

/* Sets player rendering mode. */
native set_user_rendering(index,fx = kRenderFxNone, r=255,g=255,b=255, render = kRenderNormal,amount=16);

/* Sends message to standard HL logs. */
native log_message(message[],{Float,_}:...);

/* Sends log message to specified file. */
native log_to_file(file[],message[],{Float,_}:...);

/* Returns number of players in game. */
native get_playersnum();

/* Sets indexes of players.
* Flags:
* "a" - don't collect dead players.
* "b" - don't collect alive players.
* "c" - skip bots.
* "d" - skip real players.
* "e" - match with team.
* "f" - match with part of name.
* "g" - ignore case sensitivity.
* Example: Get all alive CTs: get_players(players[32],num,"ae","CT") */
native get_players(players[32], &num ,flags[]="", team[]="");

/* Gets argument from command. */
native read_argv(id,output[],len);

/* Gets line of all arguments. */
native read_args(output[],len);

/* Returns number of arguments (+ one as command). */
native read_argc();

/* Converts string to sum of bits.
* Example: "abcd" is a sum of 1, 2, 4 and 8. */
native read_flags(flags[]);

/* Find player.
* Flags:
* "a" - with given name.
* "b" - with given part of name.
* "c" - with given authid.
* "d" - with given ip.
* "e" - with given team name.
* "f" - don't look in dead players.
* "g" - don't look in alive players.
* "h" - skip bots.
* "i" - skip real players.
* "j" - return index of last found player. 
* "k" - with given userid.
* "l" - ignore case sensitivity. */
native find_player(flags[], ... );

/* Removes quotes from sentence. */
native remove_quotes(text[]);

/* Executes command on player. */
native client_cmd(index,command[],{Float,_}:...);

/* This is an emulation of client command (command isn't send to client!).
* It allows to execute some commands on players and bots.
* Function is excellent for forcing to do action related to game (not settings!).
* The command must stand alone but in arguments you can use spaces. */
native engclient_cmd(index,command[],arg1[]="",arg2[]="");

/* Executes command on server console. */
native server_cmd(command[],{Float,_}:...);

/* Sets cvar to given value. */
native set_cvar_string(cvar[],value[]);

/* If cvar exists returns 1, in other case 0 */
native cvar_exists(cvar[]);

/* Set cvar to given float. */
native set_cvar_float(cvar[],Float:value);

/* Alias for function above but with integer value. */
stock set_cvar_num(cvarname[],value)
	set_cvar_float(cvarname,float(value))

/* Gets cvar float. */
native Float:get_cvar_float(cvarname[]);

/* Alias for function above but with integer value. */
stock get_cvar_num(cvarname[])
	return floatround(get_cvar_float(cvarname))

/* Reads cvar value. */
native get_cvar_string(cvarname[],output[],iLen);

/* Returns name of currently played map. */
native get_mapname(name[],len);

/* Returns time remaining on map in seconds. */
native get_timeleft();

/* Returns game time. */
native Float:get_gametime();

/* Returns maxplayers setting. */
native get_maxplayers();

/* Returns name of currently played mod. */
native get_modname(name[],len);

/* Returns time in given format. The most popular is: "%m/%d/%Y - %H:%M:%S". */
native get_time(format[],output[],len);

/* Calls function on specified time.
* Flags:
* "a" - repeat.
* "b" - loop task. */
native set_task(Float:time,function[],id = 0,parameter[]="",len = 0,flags[]="", repeat = 0);

/* Removes task. If outside var is set then the task can be
* removed also when was set in another plugin. */
native remove_task(id, outside=0);

/* Sets flags for player. Set flags to -1 if you want to clear all flags.
* You can use different settings by changing the id, which is from range 0 - 31. */
native set_user_flags(index,flags,id=0);

/* Gets flags from player. Set index to 0 if you want to read flags from server. */
native get_user_flags(index,id=0);

/* Removes flags for player. */
native remove_user_flags(index,flags,id=0);

/* Executes function when client uses specified command. */
native register_clcmd(client_cmd[],function[],flags = -1, info[]="");

/* Returns number of registered client commands. */
native get_clcmdsnum(flag = 0);

/* Gets name of command and additional informations. */
native get_clcmd(index, flag, command[], len1, info[], len2);

/* Executes function when specified command is used in server console. */
native register_srvcmd(server_cmd[],function[]);

/* Gets unique id of menu. */
native register_menuid(menu[]);

/* Calls function when player uses specified menu and proper keys. */
native register_menucmd(menuid,keys,function[]);

/* Gets what menu the player is watching and what keys for menu he have.
* When there is no menu the index is 0. If the id is negative then the menu
* is VGUI in other case the id is from register_menuid() function. */
native get_user_menu(index,&id,&keys);

/* Forces server to execute sent server command at current time.
* Very useful for map changes, setting cvars and other activities. */
native server_exec();

/* Emits sound. Sample must be precached. */
native emit_sound(index, channel, sample[], Float:vol, Float:att,flags, pitch);

/* Returns distance between two vectors. */
native get_distance(origin1[3],origin2[3]);

/* Registers new cvar for HL engine. */
native register_cvar(name[],string[],flags = 0,Float:float = 0.0);

/* Generates random floating point number from a to b. */
native random_float(Float:a,Float:b);

/* Generates random integer from a to b. */
native random_num(a,b);

/* Pauses function or plugin so it won't be executed.
* In most cases param1 is name of function and
* param2 name of plugin (all depends on flags).
* Flags:
* "a" - pause whole plugin.
* "b" - pause function.
* "c" - look outside the plugin (by given plugin name). */
native pause(flag[], param1[]="",param2[]="");

/* Unpauses function or plugin.
* Flags:
* "a" - unpause whole plugin.
* "b" - unpause function.
* "c" - look outside the plugin (by given plugin name). */
native unpause(flag[], func[]="",plugin[]="");

/* Returns id of client message.
* Example: get_user_msgid("TextMsg"). */
native get_user_msgid(name[]);

/* These functinos are used to generate client messages.
* You may generate menu, smoke, shockwaves, thunderlights,
* intermission and many many others messages.
* See HL SDK for more examples. */
native message_begin( dest, msg_type, origin[3]={0,0,0},player=0);
native message_end();
native write_byte( x );
native write_char( x );
native write_short( x );
native write_long( x );
native write_entity( x );
native write_angle( x );
native write_coord( x );
native write_string( x[] );

/* Gives item to player, name of item can start
* with weapon_, ammo_ and item_. This event
* is announced with proper message to all players. */
native give_item(index,item[]);

/* Sets hit zones for player. This event is announced
* with proper message to all players.
* Parts of body are as bits:
* 2   - head
* 4   - chest
* 8   - stomach
* 16  - left arm
* 32  - right arm
* 64  - left leg
* 128 - right leg*/
native set_user_hitzones(index=0,target=0,body=255);

/* Returns hit zones for player. */
native get_user_hitzones(index,target);

/* Makes that player spawns. This event is announced
* with proper message to all players. */
native user_spawn(index);

/* Creates fake client. Function returns index of player
* or 0 on failure (f.e. when maxplayers reached). */
native create_fakeclient(name[]);

/* Sets users max. speed. */
native set_user_maxspeed(index,Float:speed=-1.0);

/* Returns users max. speed. */
native Float:get_user_maxspeed(index);

/* Sets users gravity. */
native set_user_gravity(index,Float:gravity=1.0);

/* Returns users gravity. */
native Float:get_user_gravity(index);
